"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = merge;

function _bluebird() {
  const data = require("bluebird");

  _bluebird = function () {
    return data;
  };

  return data;
}

function _ramda() {
  const data = _interopRequireDefault(require("ramda"));

  _ramda = function () {
    return data;
  };

  return data;
}

function _bitId() {
  const data = require("../../../bit-id");

  _bitId = function () {
    return data;
  };

  return data;
}

function _consumer() {
  const data = require("../../../consumer");

  _consumer = function () {
    return data;
  };

  return data;
}

function _componentsList() {
  const data = _interopRequireDefault(require("../../../consumer/component/components-list"));

  _componentsList = function () {
    return data;
  };

  return data;
}

function _mergeLanes() {
  const data = require("../../../consumer/lanes/merge-lanes");

  _mergeLanes = function () {
    return data;
  };

  return data;
}

function _mergeVersion() {
  const data = require("../../../consumer/versions-ops/merge-version");

  _mergeVersion = function () {
    return data;
  };

  return data;
}

function _mergeSnaps() {
  const data = require("../../../consumer/versions-ops/merge-version/merge-snaps");

  _mergeSnaps = function () {
    return data;
  };

  return data;
}

function _generalError() {
  const data = _interopRequireDefault(require("../../../error/general-error"));

  _generalError = function () {
    return data;
  };

  return data;
}

function _hasWildcard() {
  const data = _interopRequireDefault(require("../../../utils/string/has-wildcard"));

  _hasWildcard = function () {
    return data;
  };

  return data;
}

function merge(_x, _x2, _x3, _x4, _x5, _x6, _x7, _x8) {
  return _merge.apply(this, arguments);
}

function _merge() {
  _merge = (0, _bluebird().coroutine)(function* (values, mergeStrategy, abort, resolve, idIsLane, noSnap, message, existingOnWorkspaceOnly) {
    const consumer = yield (0, _consumer().loadConsumer)();
    let mergeResults;

    const firstValue = _ramda().default.head(values);

    if (resolve) {
      mergeResults = yield (0, _mergeSnaps().resolveMerge)(consumer, values, message);
    } else if (abort) {
      mergeResults = yield (0, _mergeSnaps().abortMerge)(consumer, values);
    } else if (idIsLane) {
      if (!firstValue) throw new (_generalError().default)('please specify lane name');
      const remote = values.length === 2 ? firstValue : null;
      const laneName = values.length === 2 ? values[1] : firstValue;
      mergeResults = yield (0, _mergeLanes().mergeLanes)({
        consumer,
        remoteName: remote,
        laneName,
        mergeStrategy,
        noSnap,
        snapMessage: message,
        existingOnWorkspaceOnly
      });
    } else if (!_bitId().BitId.isValidVersion(firstValue)) {
      const bitIds = getComponentsToMerge(consumer, values); // @todo: version could be the lane only or remote/lane

      mergeResults = yield (0, _mergeSnaps().mergeComponentsFromRemote)(consumer, bitIds, mergeStrategy, noSnap, message);
    } else {
      const version = firstValue;

      const ids = _ramda().default.tail(values);

      const bitIds = getComponentsToMerge(consumer, ids);
      mergeResults = yield (0, _mergeVersion().mergeVersion)(consumer, version, bitIds, mergeStrategy);
    }

    yield consumer.onDestroy();
    return mergeResults;
  });
  return _merge.apply(this, arguments);
}

function getComponentsToMerge(consumer, ids) {
  if ((0, _hasWildcard().default)(ids)) {
    const componentsList = new (_componentsList().default)(consumer);
    return componentsList.listComponentsByIdsWithWildcard(ids);
  }

  return ids.map(id => consumer.getParsedId(id));
}