"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.tagAction = tagAction;
exports.AUTO_TAGGED_MSG = exports.NOTHING_TO_TAG_MSG = void 0;

function _bluebird() {
  const data = require("bluebird");

  _bluebird = function () {
    return data;
  };

  return data;
}

function _ramda() {
  const data = _interopRequireDefault(require("ramda"));

  _ramda = function () {
    return data;
  };

  return data;
}

function _semver() {
  const data = _interopRequireDefault(require("semver"));

  _semver = function () {
    return data;
  };

  return data;
}

function _analytics() {
  const data = require("../../../analytics/analytics");

  _analytics = function () {
    return data;
  };

  return data;
}

function _bitId() {
  const data = require("../../../bit-id");

  _bitId = function () {
    return data;
  };

  return data;
}

function _constants() {
  const data = require("../../../constants");

  _constants = function () {
    return data;
  };

  return data;
}

function _consumer() {
  const data = require("../../../consumer");

  _consumer = function () {
    return data;
  };

  return data;
}

function _componentsList() {
  const data = _interopRequireDefault(require("../../../consumer/component/components-list"));

  _componentsList = function () {
    return data;
  };

  return data;
}

function _hooks() {
  const data = _interopRequireDefault(require("../../../hooks"));

  _hooks = function () {
    return data;
  };

  return data;
}

function _hasWildcard() {
  const data = _interopRequireDefault(require("../../../utils/string/has-wildcard"));

  _hasWildcard = function () {
    return data;
  };

  return data;
}

function _semverHelper() {
  const data = require("../../../utils/semver-helper");

  _semverHelper = function () {
    return data;
  };

  return data;
}

function _loader() {
  const data = _interopRequireDefault(require("../../../cli/loader"));

  _loader = function () {
    return data;
  };

  return data;
}

const HooksManagerInstance = _hooks().default.getInstance();

const NOTHING_TO_TAG_MSG = 'nothing to tag';
exports.NOTHING_TO_TAG_MSG = NOTHING_TO_TAG_MSG;
const AUTO_TAGGED_MSG = 'auto-tagged dependents';
exports.AUTO_TAGGED_MSG = AUTO_TAGGED_MSG;

function tagAction(_x) {
  return _tagAction.apply(this, arguments);
}

function _tagAction() {
  _tagAction = (0, _bluebird().coroutine)(function* (tagParams) {
    const {
      id,
      all,
      message,
      exactVersion,
      releaseType,
      force,
      verbose,
      ignoreUnresolvedDependencies,
      ignoreNewestVersion,
      skipTests,
      scope,
      includeImported,
      skipAutoTag,
      persist
    } = tagParams;
    const idHasWildcard = (0, _hasWildcard().default)(id);
    const isAll = Boolean(all || scope || idHasWildcard);
    const validExactVersion = (0, _semverHelper().validateVersion)(exactVersion);
    const preHook = isAll ? _constants().PRE_TAG_ALL_HOOK : _constants().PRE_TAG_HOOK;
    HooksManagerInstance.triggerHook(preHook, tagParams);
    const consumer = yield (0, _consumer().loadConsumer)();
    const componentsList = new (_componentsList().default)(consumer);

    _loader().default.start('determine components to tag...');

    const newComponents = yield componentsList.listNewComponents();
    const {
      bitIds,
      warnings
    } = yield getComponentsToTag(consumer, Boolean(scope), exactVersion, includeImported, persist, force, isAll, id);
    if (_ramda().default.isEmpty(bitIds)) return null;
    const consumerTagParams = {
      ids: _bitId().BitIds.fromArray(bitIds),
      message,
      exactVersion: validExactVersion,
      releaseType,
      force,
      verbose,
      ignoreUnresolvedDependencies,
      ignoreNewestVersion,
      skipTests,
      skipAutoTag,
      persist
    };
    const tagResults = yield consumer.tag(consumerTagParams); // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!

    tagResults.warnings = warnings; // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!

    tagResults.newComponents = newComponents;
    const postHook = isAll ? _constants().POST_TAG_ALL_HOOK : _constants().POST_TAG_HOOK;
    HooksManagerInstance.triggerHook(postHook, tagResults);

    _analytics().Analytics.setExtraData('num_components', // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!
    _ramda().default.concat(tagResults.taggedComponents, tagResults.autoTaggedResults, tagResults.newComponents).length);

    yield consumer.onDestroy(); // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!

    return tagResults;
  });
  return _tagAction.apply(this, arguments);
}

function getComponentsToTag(_x2, _x3, _x4, _x5, _x6, _x7, _x8, _x9) {
  return _getComponentsToTag.apply(this, arguments);
}

function _getComponentsToTag() {
  _getComponentsToTag = (0, _bluebird().coroutine)(function* (consumer, isAllScope, exactVersion, includeImported, persist, force, isAll, id) {
    const warnings = [];
    const idHasWildcard = id && (0, _hasWildcard().default)(id);

    if (id && !idHasWildcard) {
      const bitId = consumer.getParsedId(id);

      if (!force) {
        const componentStatus = yield consumer.getComponentStatusById(bitId);
        if (componentStatus.modified === false) return {
          bitIds: [],
          warnings
        };
      }

      return {
        bitIds: [bitId],
        warnings
      };
    }

    const componentsList = new (_componentsList().default)(consumer);
    const softTaggedComponents = componentsList.listSoftTaggedComponents();

    if (!isAll && persist) {
      return {
        bitIds: softTaggedComponents,
        warnings: []
      };
    }

    const tagPendingComponents = isAllScope ? yield componentsList.listTagPendingOfAllScope(includeImported) : yield componentsList.listTagPendingComponents();

    if (isAllScope) {
      const tagPendingComponentsLatest = yield consumer.scope.latestVersions(tagPendingComponents, false);
      tagPendingComponentsLatest.forEach(componentId => {
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        if (_semver().default.gt(componentId.version, exactVersion)) {
          // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!
          warnings.push(`warning: ${componentId.toString()} has a version greater than ${exactVersion}`);
        }
      });
    }

    if (persist) {
      // add soft-tagged into the tag-pending if not already exist
      softTaggedComponents.forEach(bitId => {
        if (!tagPendingComponents.find(t => t.isEqual(bitId))) {
          softTaggedComponents.push(bitId);
        }
      });
    }

    if (idHasWildcard) {
      const bitIds = _componentsList().default.filterComponentsByWildcard(tagPendingComponents, id);

      return {
        bitIds,
        warnings
      };
    }

    return {
      bitIds: tagPendingComponents,
      warnings
    };
  });
  return _getComponentsToTag.apply(this, arguments);
}