"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.bootstrap = bootstrap;

function _bluebird() {
  const data = _interopRequireWildcard(require("bluebird"));

  _bluebird = function () {
    return data;
  };

  return data;
}

function _path() {
  const data = _interopRequireDefault(require("path"));

  _path = function () {
    return data;
  };

  return data;
}

function _chalk() {
  const data = _interopRequireDefault(require("chalk"));

  _chalk = function () {
    return data;
  };

  return data;
}

function _fsExtra() {
  const data = _interopRequireDefault(require("fs-extra"));

  _fsExtra = function () {
    return data;
  };

  return data;
}

function _semver() {
  const data = _interopRequireDefault(require("semver"));

  _semver = function () {
    return data;
  };

  return data;
}

function _analytics() {
  const data = require("./analytics/analytics");

  _analytics = function () {
    return data;
  };

  return data;
}

function _commandRunner() {
  const data = require("./cli/command-runner");

  _commandRunner = function () {
    return data;
  };

  return data;
}

function _constants() {
  const data = require("./constants");

  _constants = function () {
    return data;
  };

  return data;
}

function _hooks() {
  const data = _interopRequireDefault(require("./hooks"));

  _hooks = function () {
    return data;
  };

  return data;
}

function _logger() {
  const data = require("./logger/logger");

  _logger = function () {
    return data;
  };

  return data;
}

const MINIMUM_NODE_VERSION = '10.13.0';
process.env.MEMFS_DONT_WARN = 'true'; // suppress fs experimental warnings from memfs

require('events').EventEmitter.defaultMaxListeners = 100; // set max listeners to a more appropriate numbers

require('regenerator-runtime/runtime');

process.on('unhandledRejection', err => (0, _commandRunner().handleUnhandledRejection)(err)); // by default Bluebird enables the longStackTraces when env is `development`, or when
// BLUEBIRD_DEBUG is set.
// the drawback of enabling it all the time is a performance hit. (see http://bluebirdjs.com/docs/api/promise.longstacktraces.html)
// some commands are slower by 20% with this enabled.

_bluebird().default.config({
  longStackTraces: Boolean(process.env.BLUEBIRD_DEBUG || process.env.BIT_LOG)
});

function bootstrap() {
  return _bootstrap.apply(this, arguments);
}

function _bootstrap() {
  _bootstrap = (0, _bluebird().coroutine)(function* () {
    printBitVersionIfAsked();
    warnIfRunningAsRoot();
    verifyNodeVersionCompatibility();
    yield ensureDirectories();
    yield _analytics().Analytics.promptAnalyticsIfNeeded();

    _hooks().default.init();
  });
  return _bootstrap.apply(this, arguments);
}

function ensureDirectories() {
  return _ensureDirectories.apply(this, arguments);
}

function _ensureDirectories() {
  _ensureDirectories = (0, _bluebird().coroutine)(function* () {
    yield _fsExtra().default.ensureDir(_constants().GLOBAL_CONFIG);
    yield _fsExtra().default.ensureDir(_constants().GLOBAL_LOGS);
  });
  return _ensureDirectories.apply(this, arguments);
}

function verifyNodeVersionCompatibility() {
  const nodeVersion = process.versions.node.split('-')[0];

  const isCompatible = _semver().default.satisfies(nodeVersion, `>=${MINIMUM_NODE_VERSION}`);

  if (!isCompatible) {
    // eslint-disable-next-line no-console
    console.log(_chalk().default.red(`Node version ${nodeVersion} is not supported, please use Node.js ${MINIMUM_NODE_VERSION} or higher. If you must use legacy versions of Node.js, please use our binary installation methods. https://docs.bit.dev/docs/installation`));
    process.exit(1);
  }
}

function warnIfRunningAsRoot() {
  const isRoot = process.getuid && process.getuid() === 0;

  if (isRoot) {
    (0, _logger().printWarning)('running bit as root might cause permission issues later');
  }
}

function printBitVersionIfAsked() {
  if (process.argv[2]) {
    if (['-V', '-v', '--version'].includes(process.argv[2])) {
      const harmonyVersion = getHarmonyVersion();

      if (harmonyVersion) {
        console.log(`${harmonyVersion} (bit-bin: ${_constants().BIT_VERSION})`); // eslint-disable-line no-console
      } else {
        console.log(_constants().BIT_VERSION); // eslint-disable-line no-console
      }

      process.exit();
    }
  }
} // @todo: improve.


function getHarmonyVersion() {
  try {
    const teambitBit = require.resolve('@teambit/bit'); // eslint-disable-next-line


    const packageJson = require(_path().default.join(teambitBit, '../..', 'package.json'));

    if (packageJson.version) return packageJson.version; // this is running locally

    if (packageJson.componentId && packageJson.componentId.version) {
      return `last-tag ${packageJson.componentId.version}`;
    }

    return null;
  } catch (err) {
    return null;
  }
}