"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

require("core-js/modules/es.array.iterator");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

function _defineProperty2() {
  const data = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

  _defineProperty2 = function () {
    return data;
  };

  return data;
}

function _chalk() {
  const data = _interopRequireDefault(require("chalk"));

  _chalk = function () {
    return data;
  };

  return data;
}

function _consumer() {
  const data = require("../../../api/consumer");

  _consumer = function () {
    return data;
  };

  return data;
}

function _featureToggle() {
  const data = require("../../../api/consumer/lib/feature-toggle");

  _featureToggle = function () {
    return data;
  };

  return data;
}

function _constants() {
  const data = require("../../../constants");

  _constants = function () {
    return data;
  };

  return data;
}

class Lane {
  constructor() {
    (0, _defineProperty2().default)(this, "name", 'lane [name]');
    (0, _defineProperty2().default)(this, "description", `show lanes details
  https://${_constants().BASE_DOCS_DOMAIN}/docs/lanes`);
    (0, _defineProperty2().default)(this, "alias", '');
    (0, _defineProperty2().default)(this, "opts", [['d', 'details', 'show more details on the state of each component in each lane'], ['j', 'json', 'show lanes details in json format'], ['r', 'remote <string>', 'show remote lanes'], ['', 'merged', 'show merged lanes'], ['', 'not-merged', 'show not merged lanes']]);
    (0, _defineProperty2().default)(this, "loader", true);
    (0, _defineProperty2().default)(this, "private", true);
    (0, _defineProperty2().default)(this, "migration", true);
    (0, _defineProperty2().default)(this, "remoteOp", true);
    (0, _defineProperty2().default)(this, "skipWorkspace", true);
  }

  action([name], {
    details = false,
    remote,
    merged = false,
    notMerged = false,
    json = false
  }) {
    (0, _featureToggle().throwForUsingLaneIfDisabled)();
    return (0, _consumer().lane)({
      name,
      remote,
      showDefaultLane: json,
      merged,
      notMerged
    }).then(results => ({
      results,
      details,
      json,
      name,
      merged,
      notMerged,
      remote
    }));
  }

  report({
    results,
    details,
    json,
    name,
    merged,
    notMerged,
    remote
  }) {
    if (json) return JSON.stringify(results, null, 2);

    if (merged) {
      const mergedLanes = results.lanes.filter(l => l.isMerged);
      if (!mergedLanes.length) return _chalk().default.green('None of the lanes is merged');
      return _chalk().default.green(mergedLanes.map(m => m.name).join('\n'));
    }

    if (notMerged) {
      const unmergedLanes = results.lanes.filter(l => !l.isMerged);
      if (!unmergedLanes.length) return _chalk().default.green('All lanes are merged');
      return _chalk().default.green(unmergedLanes.map(m => m.name).join('\n'));
    }

    if (name) {
      const onlyLane = results.lanes[0];
      const title = `showing information for ${_chalk().default.bold(name)}${outputRemoteLane(onlyLane.remote)}\n`;
      const components = outputComponents(onlyLane.components);
      return title + components;
    }

    let currentLane = results.currentLane ? `current lane - ${_chalk().default.bold(results.currentLane)}` : '';

    if (details) {
      const laneDataOfCurrentLane = results.lanes.find(l => l.name === results.currentLane);
      const remoteOfCurrentLane = laneDataOfCurrentLane ? laneDataOfCurrentLane.remote : null;
      const currentLaneComponents = laneDataOfCurrentLane ? outputComponents(laneDataOfCurrentLane.components) : '';

      if (currentLane) {
        currentLane += `${outputRemoteLane(remoteOfCurrentLane)}\n${currentLaneComponents}`;
      }
    }

    const availableLanes = results.lanes.filter(l => l.name !== results.currentLane) // @ts-ignore
    .map(laneData => {
      if (details) {
        const laneTitle = `> ${_chalk().default.green(laneData.name)}${outputRemoteLane(laneData.remote)}\n`;
        const components = outputComponents(laneData.components);
        return laneTitle + components;
      }

      return `    > ${_chalk().default.green(laneData.name)} (${laneData.components.length} components)`;
    }).join('\n');
    return outputCurrentLane() + outputAvailableLanes() + outputFooter();

    function outputComponents(components) {
      const title = `\tcomponents (${components.length})\n`;
      const componentsStr = components.map(c => `\t  ${c.id.toString()} - ${c.head}`).join('\n');
      return title + componentsStr;
    }

    function outputRemoteLane(remoteLane) {
      if (!remoteLane) return '';
      return ` - (remote lane - ${remoteLane})`;
    }

    function outputCurrentLane() {
      return currentLane ? `${currentLane}\n` : '';
    }

    function outputAvailableLanes() {
      if (!availableLanes) return '';
      return remote ? `${availableLanes}\n` : `\nAvailable lanes:\n${availableLanes}\n`;
    }

    function outputFooter() {
      let footer = '\n';

      if (details) {
        footer += 'You can use --merged and --not-merged to see which of the lanes is fully merged.';
      } else {
        footer += "to get more info on all lanes in workspace use 'bit lane --details' or 'bit lane <lane-name>' for a specific lane.";
      }

      if (!remote) footer += `\nswitch lanes using 'bit switch <name>'.`;
      return footer;
    }
  }

}

exports.default = Lane;