"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

require("core-js/modules/es.array.iterator");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

function _defineProperty2() {
  const data = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

  _defineProperty2 = function () {
    return data;
  };

  return data;
}

function _chalk() {
  const data = _interopRequireDefault(require("chalk"));

  _chalk = function () {
    return data;
  };

  return data;
}

function _consumer() {
  const data = require("../../../api/consumer");

  _consumer = function () {
    return data;
  };

  return data;
}

function _constants() {
  const data = require("../../../constants");

  _constants = function () {
    return data;
  };

  return data;
}

function _generalError() {
  const data = _interopRequireDefault(require("../../../error/general-error"));

  _generalError = function () {
    return data;
  };

  return data;
}

class Untag {
  constructor() {
    (0, _defineProperty2().default)(this, "name", 'untag [id] [version]');
    (0, _defineProperty2().default)(this, "description", `revert version(s) tagged for component(s)
  For Harmony, without "--persisted" flag, it removes only the soft-tags.
  https://${_constants().BASE_DOCS_DOMAIN}/docs/tag-component-version#untagging-components
  ${(0, _constants().WILDCARD_HELP)('untag')}`);
    (0, _defineProperty2().default)(this, "alias", '');
    (0, _defineProperty2().default)(this, "opts", [['a', 'all', 'revert tag for all tagged components'], ['', 'persisted', 'harmony - revert not only the soft-tags but also the actual tags'], ['f', 'force', 'revert the tag even if used as a dependency. WARNING: components that depend on this tag will corrupt']]);
    (0, _defineProperty2().default)(this, "loader", true);
    (0, _defineProperty2().default)(this, "migration", true);
  }

  action([id, version], {
    all = false,
    force = false,
    persisted = false
  }) {
    if (!id && !all) {
      throw new (_generalError().default)('please specify a component ID or use --all flag');
    }

    if (all) {
      version = id;
      return (0, _consumer().unTagAction)(version, force, persisted);
    }

    return (0, _consumer().unTagAction)(version, force, persisted, id);
  }

  report({
    results,
    isSoftUntag
  }) {
    const titleSuffix = isSoftUntag ? 'soft-untagged (are not candidate for tagging anymore)' : 'untagged';

    const title = _chalk().default.green(`${results.length} component(s) were ${titleSuffix}:\n`);

    const softTagSuggestion = isSoftUntag ? _chalk().default.bold('\nto actually remove the local tags, use --persisted flag') : '';
    const components = results.map(result => {
      return `${_chalk().default.cyan(result.id.toStringWithoutVersion())}. version(s): ${result.versions.join(', ')}`;
    });
    return title + components.join('\n') + softTagSuggestion;
  }

}

exports.default = Untag;