import { Dependency } from '..';
import { BitId } from '../../../../bit-id';
import Consumer from '../../../../consumer/consumer';
import { PathLinux, PathLinuxRelative, PathOsBased } from '../../../../utils/path';
import ComponentMap from '../../../bit-map/component-map';
import Component from '../../../component/consumer-component';
import { RelativePath } from '../dependency';
import { FileObject, ImportSpecifier, Tree } from '../files-dependency-builder/types/dependency-tree-type';
import OverridesDependencies from './overrides-dependencies';
import { ResolvedPackageData } from '../../../../utils/packages';
export declare type AllDependencies = {
    dependencies: Dependency[];
    devDependencies: Dependency[];
};
export declare type AllPackagesDependencies = {
    packageDependencies: Record<string, any> | null | undefined;
    devPackageDependencies: Record<string, any> | null | undefined;
    compilerPackageDependencies: Record<string, any> | null | undefined;
    testerPackageDependencies: Record<string, any> | null | undefined;
    peerPackageDependencies: Record<string, any> | null | undefined;
};
export declare type FileType = {
    isTestFile: boolean;
};
interface UntrackedFileEntry {
    relativePath: string;
    existing: boolean;
}
export interface UntrackedFileDependencyEntry {
    nested: boolean;
    untrackedFiles: Array<UntrackedFileEntry>;
}
export declare type RelativeComponentsAuthoredEntry = {
    importSource: string;
    componentId: BitId;
    relativePath: RelativePath;
};
declare type UntrackedDependenciesIssues = Record<string, UntrackedFileDependencyEntry>;
declare type RelativeComponentsAuthoredIssues = {
    [fileName: string]: RelativeComponentsAuthoredEntry[];
};
export declare type Issues = {
    missingPackagesDependenciesOnFs: {};
    missingPackagesDependenciesFromOverrides: string[];
    missingComponents: {};
    untrackedDependencies: UntrackedDependenciesIssues;
    missingDependenciesOnFs: {};
    missingLinks: {};
    missingCustomModuleResolutionLinks: {};
    customModuleResolutionUsed: {};
    relativeComponents: {};
    relativeComponentsAuthored: RelativeComponentsAuthoredIssues;
    parseErrors: {};
    resolveErrors: {};
    missingBits: {};
};
declare type WorkspacePolicyGetter = () => {
    dependencies?: Record<string, string>;
    peerDependencies?: Record<string, string>;
};
export default class DependencyResolver {
    component: Component;
    consumer: Consumer;
    componentId: BitId;
    componentMap: ComponentMap;
    componentFromModel: Component;
    consumerPath: PathOsBased;
    tree: Tree;
    allDependencies: AllDependencies;
    allPackagesDependencies: AllPackagesDependencies;
    issues: Issues;
    processedFiles: string[];
    compilerFiles: PathLinux[];
    testerFiles: PathLinux[];
    overridesDependencies: OverridesDependencies;
    static getWorkspacePolicy: WorkspacePolicyGetter;
    static registerWorkspacePolicyGetter(func: WorkspacePolicyGetter): void;
    static getDepResolverAspectName: () => string;
    static getCoreAspectsPackagesAndIds: () => Record<string, string>;
    constructor(component: Component, consumer: Consumer, componentId: BitId);
    setTree(tree: Tree): void;
    loadDependenciesForComponent(bitDir: string, cacheResolvedDependencies: Record<string, any>, cacheProjectAst: Record<string, any> | undefined): Promise<Component>;
    populateDependencies(files: string[], testsFiles: string[]): void;
    addCustomResolvedIssues(): void;
    removeIgnoredPackagesByOverrides(): void;
    throwForNonExistFile(file: string): void;
    manuallyAddDependencies(): void;
    applyOverridesOnEnvPackages(): void;
    traverseTreeForComponentId(depFile: PathLinux): BitId | undefined;
    getComponentIdByResolvedPackageData(bit: ResolvedPackageData): BitId;
    getComponentIdByDepFile(depFile: PathLinux): {
        componentId: BitId | null | undefined;
        depFileRelative: PathLinux;
        destination: string | null | undefined;
    };
    _getComponentIdFromCustomResolveToPackageWithDist(depFile: string): BitId | null | undefined;
    getDependencyPathsFromModel(componentId: BitId, depFile: PathLinux, rootDir: PathLinux): {
        componentId: BitId;
        destination: string;
        depFileRelative: string;
    };
    processDepFiles(originFile: PathLinuxRelative, fileType: FileType, nested?: boolean): void;
    processOneDepFile(originFile: PathLinuxRelative, depFile: string, importSpecifiers: ImportSpecifier[] | undefined, linkFile: string | undefined, fileType: FileType, depFileObject: FileObject, nested?: boolean): boolean;
    processLinkFile(originFile: PathLinuxRelative, linkFile: FileObject, fileType: FileType): void;
    processBits(originFile: PathLinuxRelative, fileType: FileType): void;
    getValidVersion(version: string | undefined): string | null;
    processPackages(originFile: PathLinuxRelative, fileType: FileType): void;
    processMissing(originFile: PathLinuxRelative, fileType: FileType): void;
    _addToMissingComponentsIfNeeded(missingComponents: string[], originFile: string, fileType: FileType): void;
    processErrors(originFile: PathLinuxRelative): void;
    processCoreAspects(originFile: PathLinuxRelative): void;
    pushCoreAspectsToDependencyResolver(usedCoreAspects: string[]): void;
    processUnidentifiedPackages(originFile: PathLinuxRelative, fileType: FileType): void;
    private _pushToDependenciesIfNotExist;
    pushToDependenciesArray(currentComponentsDeps: Dependency, fileType: FileType): void;
    pushToDependencyResolverExtension(dataFiled: string, data: any, operation?: 'add' | 'set'): void;
    removeDevAndEnvDepsIfTheyAlsoRegulars(): void;
    findOriginallyCustomModuleResolvedDependencies(packages: string[]): Record<string, any> | null | undefined;
    combineIssues(): void;
    removeEmptyIssues(): void;
    getExistingDependency(dependencies: Dependency[], id: BitId): Dependency | null | undefined;
    getExistingDepRelativePaths(dependency: Dependency, relativePath: RelativePath): RelativePath | null | undefined;
    getDiffSpecifiers(originSpecifiers: ImportSpecifier[], targetSpecifiers: ImportSpecifier[]): any;
    populatePeerPackageDependencies(): void;
    applyWorkspacePolicy(): void;
    _getPackageJson(): Record<string, any> | undefined;
    _addTypesPackagesForTypeScript(packages: Record<string, any>, originFile: PathLinuxRelative): void;
    _pkgFieldMapping(field: string): "packageDependencies" | "devPackageDependencies" | "peerPackageDependencies";
    _pushToUntrackDependenciesIssues(originFile: any, depFileRelative: any, nested?: boolean): void;
    _pushToRelativeComponentsIssues(originFile: any, componentId: any): void;
    _pushToRelativeComponentsAuthoredIssues(originFile: any, componentId: any, importSource: string, relativePath: RelativePath): void;
    _pushToMissingBitsIssues(originFile: PathLinuxRelative, componentId: BitId): void;
    _pushToMissingDependenciesOnFs(originFile: PathLinuxRelative, missingFiles: string[]): void;
    _pushToMissingPackagesDependenciesIssues(originFile: PathLinuxRelative, missingPackages: string[]): void;
    _pushToMissingCustomModuleIssues(originFile: PathLinuxRelative, componentId: BitId): void;
    _pushToMissingLinksIssues(originFile: PathLinuxRelative, componentId: BitId): void;
    _pushToMissingComponentsIssues(originFile: PathLinuxRelative, componentId: BitId): void;
}
export {};
