"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.MissingHandler = void 0;

function _ramda() {
  const data = _interopRequireDefault(require("ramda"));

  _ramda = function () {
    return data;
  };

  return data;
}

function _constants() {
  const data = require("../../../../constants");

  _constants = function () {
    return data;
  };

  return data;
}

function _packages() {
  const data = require("../../../../utils/packages");

  _packages = function () {
    return data;
  };

  return data;
}

function _generateTreeMadge() {
  const data = require("./generate-tree-madge");

  _generateTreeMadge = function () {
    return data;
  };

  return data;
}

class MissingHandler {
  constructor(missing, componentDir, workspacePath, bindingPrefix, isLegacyProject) {
    this.missing = missing;
    this.componentDir = componentDir;
    this.workspacePath = workspacePath;
    this.bindingPrefix = bindingPrefix;
    this.isLegacyProject = isLegacyProject;
  }
  /**
   * Run over each entry in the missing array and transform the missing from list of paths
   * to object with missing types
   */


  groupAndFindMissing() {
    const missingGroups = this.groupMissingByType();
    missingGroups.forEach(group => {
      if (group.packages) group.packages = group.packages.map(_packages().resolvePackageNameByPath);
      if (group.bits) group.bits = group.bits.map(_packages().resolvePackageNameByPath);
    }); // This is a hack to solve problems that madge has with packages for type script files
    // It see them as missing even if they are exists

    const foundPackages = {
      packages: {},
      bits: []
    };
    missingGroups.forEach(group => this.processMissingGroup(group, foundPackages));
    return {
      missingGroups,
      foundPackages
    };
  }

  processMissingGroup(group, foundPackages) {
    if (!group.packages) return;
    const missingPackages = [];
    group.packages.forEach(packageName => {
      // Don't try to resolve the same package twice
      if (missingPackages.includes(packageName)) return;
      const resolvedPath = (0, _packages().resolvePackagePath)(packageName, this.componentDir, this.workspacePath);

      if (!resolvedPath) {
        missingPackages.push(packageName);
        return;
      }

      const resolvedPackageData = (0, _packages().resolvePackageData)(this.componentDir, resolvedPath);

      if (!resolvedPackageData) {
        missingPackages.push(packageName);
        return;
      } // if the package is actually a component add it to the components (bits) list


      if (resolvedPackageData.componentId) {
        foundPackages.bits.push(resolvedPackageData);
      } else {
        const version = resolvedPackageData.versionUsedByDependent || resolvedPackageData.concreteVersion;
        if (!version) throw new Error(`unable to find the version for a package ${packageName}`);
        const packageWithVersion = {
          [resolvedPackageData.name]: version
        };
        Object.assign(foundPackages.packages, packageWithVersion);
      }
    });
  }
  /**
   * Group missing dependencies by types (files, bits, packages)
   * @param {Array} missing list of missing paths to group
   * @returns {Function} function which group the dependencies
   */


  groupMissingByType() {
    const byPathType = _ramda().default.groupBy(item => {
      if (this.isLegacyProject && (item.startsWith(`${this.bindingPrefix}/`) || item.startsWith(`${_constants().DEFAULT_BINDINGS_PREFIX}/`))) {
        return 'bits';
      }

      return item.startsWith('.') ? 'files' : 'packages';
    });

    return Object.keys(this.missing).map(key => Object.assign({
      originFile: (0, _generateTreeMadge().processPath)(key, {}, this.componentDir)
    }, byPathType(this.missing[key])));
  }

}

exports.MissingHandler = MissingHandler;