import { Scope } from '../../../scope';
import { Source } from '../../../scope/models';
import { Ref } from '../../../scope/objects';
import { ExtensionDataList } from '../../config';
import { ArtifactVinyl } from './artifact';
export declare type ArtifactRef = {
    relativePath: string;
    ref: Ref;
};
export declare type ArtifactModel = {
    relativePath: string;
    file: string;
};
export declare type ArtifactSource = {
    relativePath: string;
    source: Source;
};
export declare type ArtifactObject = {
    name: string;
    description?: string;
    generatedBy: string;
    storage: string;
    task: {
        id: string;
        name?: string;
    };
    files: ArtifactFiles;
};
export declare class ArtifactFiles {
    paths: string[];
    vinyls: ArtifactVinyl[];
    refs: ArtifactRef[];
    constructor(paths?: string[], vinyls?: ArtifactVinyl[], refs?: ArtifactRef[]);
    clone(): ArtifactFiles;
    populateRefsFromSources(sources: ArtifactSource[]): void;
    populateVinylsFromPaths(rootDir: string): void;
    getExistingVinyls(): ArtifactVinyl[];
    isEmpty(): boolean;
    static fromModel(artifactModels?: ArtifactModel[]): ArtifactFiles;
    static fromVinylsToSources(vinyls: ArtifactVinyl[]): ArtifactSource[];
    getVinylsAndImportIfMissing(scopeName: string, scope: Scope): Promise<ArtifactVinyl[]>;
}
export declare function refsToModelObjects(refs: ArtifactRef[]): ArtifactModel[];
export declare function getRefsFromExtensions(extensions: ExtensionDataList): Ref[];
export declare function getArtifactFilesByExtension(extensions: ExtensionDataList, extensionName: string): ArtifactFiles | undefined;
export declare function convertBuildArtifactsToModelObject(extensions: ExtensionDataList): void;
export declare function convertBuildArtifactsFromModelObject(extensions: ExtensionDataList): void;
export declare function getArtifactsFiles(extensions: ExtensionDataList): ArtifactFiles[];
export declare function reStructureBuildArtifacts(extensions: ExtensionDataList): void;
export declare function deserializeArtifactFiles(obj: {
    paths: string[];
    vinyls: ArtifactVinyl[];
    refs: ArtifactRef[];
}): ArtifactFiles;
