"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

require("core-js/modules/es.array.iterator");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ExtensionDataList = exports.ExtensionDataEntry = void 0;

function _defineProperty2() {
  const data = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

  _defineProperty2 = function () {
    return data;
  };

  return data;
}

function _ramda() {
  const data = _interopRequireWildcard(require("ramda"));

  _ramda = function () {
    return data;
  };

  return data;
}

function _ramdaAdjunct() {
  const data = require("ramda-adjunct");

  _ramdaAdjunct = function () {
    return data;
  };

  return data;
}

function _bitId() {
  const data = require("../../bit-id");

  _bitId = function () {
    return data;
  };

  return data;
}

function _utils() {
  const data = require("../../utils");

  _utils = function () {
    return data;
  };

  return data;
}

function _artifactFiles() {
  const data = require("../component/sources/artifact-files");

  _artifactFiles = function () {
    return data;
  };

  return data;
}

/* eslint-disable max-classes-per-file */
const mergeReducer = (accumulator, currentValue) => _ramda().default.unionWith(ignoreVersionPredicate, accumulator, currentValue);

class ExtensionDataEntry {
  constructor(legacyId, extensionId, name, config = {}, data = {}, newExtensionId = undefined) {
    this.legacyId = legacyId;
    this.extensionId = extensionId;
    this.name = name;
    this.config = config;
    this.data = data;
    this.newExtensionId = newExtensionId;
  }

  get id() {
    if (this.extensionId) return this.extensionId;
    if (this.name) return this.name;
    if (this.legacyId) return this.legacyId;
    return '';
  }

  get stringId() {
    var _this$extensionId;

    if (this.extensionId) return (_this$extensionId = this.extensionId) === null || _this$extensionId === void 0 ? void 0 : _this$extensionId.toString();
    if (this.name) return this.name;
    if (this.legacyId) return this.legacyId;
    return '';
  }

  get isLegacy() {
    var _this$config;

    if ((_this$config = this.config) === null || _this$config === void 0 ? void 0 : _this$config.__legacy) return true;
    return false;
  }

  clone() {
    var _this$extensionId2;

    return new ExtensionDataEntry(this.legacyId, (_this$extensionId2 = this.extensionId) === null || _this$extensionId2 === void 0 ? void 0 : _this$extensionId2.clone(), this.name, _ramda().default.clone(this.config), _ramda().default.clone(this.data));
  } // static fromConfigEntry(id: BitId, config: Record<string, any>) {
  //   // TODO: refactor the core names registry to be outside the ExtensionDataList
  //   // eslint-disable-next-line @typescript-eslint/no-use-before-define
  //   const isCore = ExtensionDataList.coreExtensionsNames.has(id.toString());
  //   let entry;
  //   if (!isCore) {
  //     entry = new ExtensionDataEntry(undefined, id, undefined, config, undefined);
  //   } else {
  //     entry = new ExtensionDataEntry(undefined, undefined, id.toString(), config, undefined);
  //   }
  //   return entry;
  // }


}

exports.ExtensionDataEntry = ExtensionDataEntry;

class ExtensionDataList extends Array {
  static registerCoreExtensionName(name) {
    ExtensionDataList.coreExtensionsNames.set(name, '');
  }

  static registerManyCoreExtensionNames(names) {
    names.forEach(name => {
      ExtensionDataList.coreExtensionsNames.set(name, '');
    });
  }

  get ids() {
    const list = this.map(entry => entry.stringId);
    return list;
  }
  /**
   * returns only new 3rd party extension ids, not core, nor legacy.
   */


  get extensionsBitIds() {
    const bitIds = this.filter(entry => entry.extensionId).map(entry => entry.extensionId);
    return _bitId().BitIds.fromArray(bitIds);
  }

  toModelObjects() {
    const extensionsClone = this.clone();
    extensionsClone.forEach(ext => {
      if (ext.extensionId) {
        // TODO: fix the types of extensions. after this it should be an object not an object id
        // @ts-ignore
        ext.extensionId = ext.extensionId.serialize();
      }
    });
    (0, _artifactFiles().convertBuildArtifactsToModelObject)(extensionsClone);
    return extensionsClone;
  }

  static fromModelObject(entries) {
    const extensionDataList = ExtensionDataList.fromArray(entries);
    (0, _artifactFiles().convertBuildArtifactsFromModelObject)(extensionDataList);
    return extensionDataList;
  }

  findExtension(extensionId, ignoreVersion = false, ignoreScope = false) {
    if (ExtensionDataList.coreExtensionsNames.has(extensionId)) {
      return this.findCoreExtension(extensionId);
    }

    return this.find(extEntry => {
      if (ignoreVersion && ignoreScope) {
        var _extEntry$extensionId;

        return ((_extEntry$extensionId = extEntry.extensionId) === null || _extEntry$extensionId === void 0 ? void 0 : _extEntry$extensionId.toStringWithoutScopeAndVersion()) === extensionId;
      }

      if (ignoreVersion) {
        var _extEntry$extensionId2;

        return ((_extEntry$extensionId2 = extEntry.extensionId) === null || _extEntry$extensionId2 === void 0 ? void 0 : _extEntry$extensionId2.toStringWithoutVersion()) === extensionId;
      }

      if (ignoreScope) {
        var _extEntry$extensionId3;

        return ((_extEntry$extensionId3 = extEntry.extensionId) === null || _extEntry$extensionId3 === void 0 ? void 0 : _extEntry$extensionId3.toStringWithoutScope()) === extensionId;
      }

      return extEntry.stringId === extensionId;
    });
  }

  findCoreExtension(extensionId) {
    return this.find(extEntry => extEntry.name === extensionId);
  }

  remove(id) {
    return ExtensionDataList.fromArray(this.filter(entry => {
      return entry.stringId !== id.toString() && entry.stringId !== id.toStringWithoutVersion();
    }));
  }

  toConfigObject() {
    const res = {};
    this.forEach(entry => {
      if (entry.config && !_ramda().default.isEmpty(entry.config)) {
        res[entry.stringId] = entry.config;
      }
    });
    return res;
  }

  toConfigArray() {
    const arr = this.map(entry => {
      // Remove extensions without config
      if (entry.config && !_ramda().default.isEmpty(entry.config)) {
        return {
          id: entry.stringId,
          config: entry.config
        };
      }

      return undefined;
    });
    return (0, _ramdaAdjunct().compact)(arr);
  }

  clone() {
    const extensionDataEntries = this.map(extensionData => extensionData.clone());
    const extensionDataList = new ExtensionDataList(...extensionDataEntries);
    (0, _artifactFiles().reStructureBuildArtifacts)(extensionDataList);
    return extensionDataList;
  }

  _filterLegacy() {
    return ExtensionDataList.fromArray(this.filter(ext => !ext.isLegacy));
  }

  sortById() {
    const arr = _ramda().default.sortBy(_ramda().default.prop('stringId'), this); // Also sort the config


    arr.forEach(entry => {
      entry.config = (0, _utils().sortObject)(entry.config);
    });
    return ExtensionDataList.fromArray(arr);
  }

  static fromConfigObject(obj) {
    const arr = [];
    (0, _ramda().forEachObjIndexed)((config, id) => {
      const isCore = ExtensionDataList.coreExtensionsNames.has(id);
      let entry;

      if (!isCore) {
        const parsedId = _bitId().BitId.parse(id, true);

        entry = new ExtensionDataEntry(undefined, parsedId, undefined, config, undefined);
      } else {
        entry = new ExtensionDataEntry(undefined, undefined, id, config, undefined);
      }

      arr.push(entry);
    }, obj);
    return this.fromArray(arr);
  }

  static fromArray(entries) {
    if (!entries || !entries.length) {
      return new ExtensionDataList();
    }

    return new ExtensionDataList(...entries);
  }
  /**
   * Merge a list of ExtensionDataList into one ExtensionDataList
   * In case of entry with the same id appear in more than one list
   * the former in the list will be taken
   * see unit tests for examples
   *
   * Make sure you extension ids are resolved before call this, otherwise you might get unexpected results
   * for example:
   * you might have 2 entries like: default-scope/my-extension and my-extension on the same time
   *
   * @static
   * @param {ExtensionDataList[]} list
   * @returns {ExtensionDataList}
   * @memberof ExtensionDataList
   */


  static mergeConfigs(list) {
    if (list.length === 1) {
      return list[0];
    }

    const merged = list.reduce(mergeReducer, new ExtensionDataList());
    return ExtensionDataList.fromArray(merged);
  }

}

exports.ExtensionDataList = ExtensionDataList;
(0, _defineProperty2().default)(ExtensionDataList, "coreExtensionsNames", new Map());

function ignoreVersionPredicate(extensionEntry1, extensionEntry2) {
  if (extensionEntry1.extensionId && extensionEntry2.extensionId) {
    return extensionEntry1.extensionId.isEqualWithoutVersion(extensionEntry2.extensionId);
  }

  if (extensionEntry1.name && extensionEntry2.name) {
    return extensionEntry1.name === extensionEntry2.name;
  }

  return false;
}