"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

require("core-js/modules/es.array.iterator");

require("core-js/modules/es.promise");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.updateLanesAfterExport = updateLanesAfterExport;
exports.getLaneCompIdsToExport = getLaneCompIdsToExport;
exports.isUserTryingToExportLanes = isUserTryingToExportLanes;

function _bluebird() {
  const data = require("bluebird");

  _bluebird = function () {
    return data;
  };

  return data;
}

function _ramda() {
  const data = _interopRequireDefault(require("ramda"));

  _ramda = function () {
    return data;
  };

  return data;
}

function _bitId() {
  const data = require("../../bit-id");

  _bitId = function () {
    return data;
  };

  return data;
}

function _loader() {
  const data = _interopRequireDefault(require("../../cli/loader"));

  _loader = function () {
    return data;
  };

  return data;
}

function _loaderMessages() {
  const data = require("../../cli/loader/loader-messages");

  _loaderMessages = function () {
    return data;
  };

  return data;
}

function _generalError() {
  const data = _interopRequireDefault(require("../../error/general-error"));

  _generalError = function () {
    return data;
  };

  return data;
}

function _laneId() {
  const data = _interopRequireDefault(require("../../lane-id/lane-id"));

  _laneId = function () {
    return data;
  };

  return data;
}

function _workspaceLane() {
  const data = _interopRequireDefault(require("../bit-map/workspace-lane"));

  _workspaceLane = function () {
    return data;
  };

  return data;
}

function _componentsList() {
  const data = _interopRequireDefault(require("../component/components-list"));

  _componentsList = function () {
    return data;
  };

  return data;
}

function updateLanesAfterExport(_x, _x2) {
  return _updateLanesAfterExport.apply(this, arguments);
}

function _updateLanesAfterExport() {
  _updateLanesAfterExport = (0, _bluebird().coroutine)(function* (consumer, lanes) {
    const lanesToUpdate = lanes.filter(l => l.remoteLaneId); // lanes that don't have remoteLaneId should not be updated. it happens when updating to a
    // different remote with no intention to save the remote.

    if (!lanesToUpdate.length) return;
    const currentLane = consumer.getCurrentLaneId();
    const workspaceLanesToUpdate = [];
    lanesToUpdate.forEach(lane => {
      const remoteLaneId = lane.remoteLaneId;
      consumer.scope.lanes.trackLane({
        localLane: lane.name,
        remoteLane: remoteLaneId.name,
        remoteScope: remoteLaneId.scope
      });
      const isCurrentLane = lane.name === currentLane.name;

      if (isCurrentLane) {
        consumer.bitMap.setRemoteLane(remoteLaneId);
      }

      const workspaceLane = isCurrentLane ? consumer.bitMap.workspaceLane : // bitMap.workspaceLane is empty only when is on master
      _workspaceLane().default.load(lane.name, consumer.scope.path);
      if (!isCurrentLane) workspaceLanesToUpdate.push(workspaceLane);
      consumer.bitMap.updateLanesProperty(workspaceLane, remoteLaneId);
      workspaceLane.reset();
    });
    yield Promise.all(workspaceLanesToUpdate.map(l => l.write()));
  });
  return _updateLanesAfterExport.apply(this, arguments);
}

function getLaneCompIdsToExport(_x3, _x4, _x5) {
  return _getLaneCompIdsToExport.apply(this, arguments);
}

function _getLaneCompIdsToExport() {
  _getLaneCompIdsToExport = (0, _bluebird().coroutine)(function* (consumer, ids, includeNonStaged) {
    const currentLaneId = consumer.getCurrentLaneId();
    const laneIds = ids.length ? ids.map(laneName => new (_laneId().default)({
      name: laneName
    })) : [currentLaneId];
    const nonExistingLanes = [];
    const lanesObjects = [];
    yield Promise.all(laneIds.map( /*#__PURE__*/function () {
      var _ref = (0, _bluebird().coroutine)(function* (laneId) {
        const laneObject = yield consumer.scope.loadLane(laneId);

        if (laneObject) {
          lanesObjects.push(laneObject);
        } else if (!laneId.isDefault()) {
          nonExistingLanes.push(laneId.name);
        }
      });

      return function (_x6) {
        return _ref.apply(this, arguments);
      };
    }()));

    if (nonExistingLanes.length) {
      throw new (_generalError().default)(`unable to export the following lanes ${nonExistingLanes.join(', ')}. they don't exist or are empty`);
    }

    _loader().default.start(_loaderMessages().BEFORE_LOADING_COMPONENTS);

    const componentsList = new (_componentsList().default)(consumer);
    const compsToExportP = lanesObjects.map( /*#__PURE__*/function () {
      var _ref2 = (0, _bluebird().coroutine)(function* (laneObject) {
        // null in case of default-lane
        return includeNonStaged ? componentsList.listNonNewComponentsIds() : componentsList.listExportPendingComponentsIds(laneObject);
      });

      return function (_x7) {
        return _ref2.apply(this, arguments);
      };
    }());

    const componentsToExport = _bitId().BitIds.fromArray(_ramda().default.flatten(yield Promise.all(compsToExportP)));

    return {
      componentsToExport,
      lanesObjects
    };
  });
  return _getLaneCompIdsToExport.apply(this, arguments);
}

function isUserTryingToExportLanes(consumer, ids, lanes) {
  if (!ids.length) {
    const currentLaneId = consumer.getCurrentLaneId(); // if no ids entered, when a user checked out to a lane, we should export the lane

    return !currentLaneId.isDefault();
  }

  return lanes;
} // leave this here in case we do want to guess whether a user wants to export a lane.
// export async function isUserTryingToExportLanes(consumer: Consumer) {
//   const getLaneNames = async (): Promise<string[]> => {
//     const lanesObj = await consumer.scope.listLanes();
//     const laneNames = lanesObj.map(lane => lane.name);
//     laneNames.push(DEFAULT_LANE);
//     return laneNames;
//   };
//   const laneNames = await getLaneNames();
//   const idsFromWorkspaceAndScope = await componentsList.listAllIdsFromWorkspaceAndScope();
//   const currentLaneId = consumer.getCurrentLaneId();
//   if (lanes) return true;
//   if (!ids.length) {
//     // if no ids entered, when a user checked out to a lane, we should export the lane
//     return !currentLaneId.isDefault();
//   }
//   if (ids.every(id => !laneNames.includes(id))) {
//     // if none of the ids is lane, then user is not trying to export lanes
//     return false;
//   }
//   // some or all ids are lane names, if all are not ids, user is trying to export lanes
//   return ids.every(id => {
//     if (laneNames.includes(id) && idsFromWorkspaceAndScope.hasWithoutScopeAndVersionAsString(id)) {
//       throw new GeneralError(`the id ${id} is both, a component-name and a lane-name`);
//     }
//     return laneNames.includes(id);
//   });
// };