"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

require("core-js/modules/es.symbol.description");

function _bluebird() {
  const data = require("bluebird");

  _bluebird = function () {
    return data;
  };

  return data;
}

function _chai() {
  const data = require("chai");

  _chai = function () {
    return data;
  };

  return data;
}

function _fsExtra() {
  const data = _interopRequireDefault(require("fs-extra"));

  _fsExtra = function () {
    return data;
  };

  return data;
}

function path() {
  const data = _interopRequireWildcard(require("path"));

  path = function () {
    return data;
  };

  return data;
}

function _() {
  const data = _interopRequireDefault(require("./"));

  _ = function () {
    return data;
  };

  return data;
}

const fixtures = path().join(__dirname, '../../..', 'fixtures', 'jsdoc', 'vue');
describe('Vue docs Parser', () => {
  describe('Invalid code', () => {
    it('should returns an empty object', /*#__PURE__*/(0, _bluebird().coroutine)(function* () {
      const doclets = (yield (0, _().default)('this is an invalid code'))[0];
      (0, _chai().expect)(doclets).to.be.a('object');
      (0, _chai().expect)(doclets).to.have.property('methods').to.be.a('array').to.have.lengthOf(0);
      (0, _chai().expect)(doclets).to.have.property('properties').to.be.a('array').to.have.lengthOf(0);
    }));
  });
  describe('parse()', () => {
    let doclet;
    before( /*#__PURE__*/(0, _bluebird().coroutine)(function* () {
      const componentFile = path().join(fixtures, 'checkbox.vue');

      const componentFileContent = _fsExtra().default.readFileSync(componentFile).toString();

      doclet = (yield (0, _().default)(componentFileContent, componentFile))[0];
    }));
    describe('name and description', () => {
      it('should recognize the component name', () => {
        (0, _chai().expect)(doclet.name).to.equal('checkbox');
      });
      it('should recognize the description', () => {
        (0, _chai().expect)(doclet.description).to.equal('A simple checkbox component');
      });
    });
    describe('Method Declaration', () => {
      it('should recognize the check method', () => {
        const methodDef = doclet.methods[0];
        (0, _chai().expect)(methodDef).to.have.property('name').that.equals('check');
        (0, _chai().expect)(methodDef).to.have.property('description').that.equals('Check the checkbox');
        (0, _chai().expect)(methodDef).to.have.property('access').that.equals('public');
        (0, _chai().expect)(methodDef).to.have.property('returns').that.is.an('object').that.is.not.empty; // TODO: add tests for this
        // expect(methodDef)
        //   .to.have.property('examples')
        //   .that.is.an('array').that.is.empty;
        // expect(methodDef)
        //   .to.have.property('args')
        //   .that.is.an('array').that.is.empty;
      });
      it('should recognize the uncommentedMethod method', () => {
        const methodDef = doclet.methods[1];
        (0, _chai().expect)(methodDef).to.have.property('name').that.equals('uncommentedMethod');
        (0, _chai().expect)(methodDef).to.have.property('description').that.is.empty;
      });
    });
    describe('Properties property', () => {
      it('should parse the properties correctly', () => {
        (0, _chai().expect)(doclet).to.have.property('properties').that.is.an('array').that.have.lengthOf(5);
        (0, _chai().expect)(doclet.properties[0].name).to.equal('model');
        (0, _chai().expect)(doclet.properties[0].type).to.equal('Array');
        (0, _chai().expect)(doclet.properties[0].description).to.equal('The checkbox model');
        (0, _chai().expect)(doclet.properties[0].required).to.equal(true);
        (0, _chai().expect)(doclet.properties[0].defaultValue.value).to.be.undefined;
        (0, _chai().expect)(doclet.properties[0].defaultValue.computed).to.equal(false);
        (0, _chai().expect)(doclet.properties[1].name).to.equal('disabled');
        (0, _chai().expect)(doclet.properties[1].type).to.equal('Boolean');
        (0, _chai().expect)(doclet.properties[1].description).to.equal('Initial checkbox state');
        (0, _chai().expect)(doclet.properties[1].required).to.equal(false);
        (0, _chai().expect)(doclet.properties[1].defaultValue.value).to.be.undefined;
        (0, _chai().expect)(doclet.properties[1].defaultValue.computed).to.equal(false);
        (0, _chai().expect)(doclet.properties[2].name).to.equal('checked');
        (0, _chai().expect)(doclet.properties[2].type).to.equal('Boolean');
        (0, _chai().expect)(doclet.properties[2].description).to.equal('Initial checkbox value');
        (0, _chai().expect)(doclet.properties[2].required).to.equal(false);
        (0, _chai().expect)(doclet.properties[2].defaultValue.value).to.be.true;
        (0, _chai().expect)(doclet.properties[2].defaultValue.computed).to.equal(false);
        (0, _chai().expect)(doclet.properties[3].name).to.equal('prop-with-camel');
        (0, _chai().expect)(doclet.properties[3].type).to.equal('Object');
        (0, _chai().expect)(doclet.properties[3].description).to.equal('Prop with camel name');
        (0, _chai().expect)(doclet.properties[3].required).to.equal(false);
        (0, _chai().expect)(doclet.properties[3].defaultValue.value).to.be.equal("() => ({ name: 'X'})");
        (0, _chai().expect)(doclet.properties[3].defaultValue.computed).to.equal(false);
        (0, _chai().expect)(doclet.properties[4].name).to.equal('id');
        (0, _chai().expect)(doclet.properties[4].type).to.be.undefined;
        (0, _chai().expect)(doclet.properties[4].description).to.equal('computed id');
        (0, _chai().expect)(doclet.properties[4].required).to.equal(false);
        (0, _chai().expect)(doclet.properties[4].defaultValue.value).to.be.null;
        (0, _chai().expect)(doclet.properties[4].defaultValue.computed).to.equal(true);
      });
    });
  });
});