"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

require("core-js/modules/es.array.iterator");

require("core-js/modules/es.promise");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

function _bluebird() {
  const data = require("bluebird");

  _bluebird = function () {
    return data;
  };

  return data;
}

function _defineProperty2() {
  const data = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

  _defineProperty2 = function () {
    return data;
  };

  return data;
}

function _constants() {
  const data = require("../../constants");

  _constants = function () {
    return data;
  };

  return data;
}

function _generalError() {
  const data = _interopRequireDefault(require("../../error/general-error"));

  _generalError = function () {
    return data;
  };

  return data;
}

function _laneId() {
  const data = _interopRequireWildcard(require("../../lane-id/lane-id"));

  _laneId = function () {
    return data;
  };

  return data;
}

function _objects() {
  const data = require("../objects");

  _objects = function () {
    return data;
  };

  return data;
}

function _componentsIndex() {
  const data = require("../objects/components-index");

  _componentsIndex = function () {
    return data;
  };

  return data;
}

class Lanes {
  constructor(objects, scopeJson) {
    (0, _defineProperty2().default)(this, "objects", void 0);
    (0, _defineProperty2().default)(this, "scopeJson", void 0);
    this.objects = objects;
    this.scopeJson = scopeJson;
  }

  listLanes() {
    var _this = this;

    return (0, _bluebird().coroutine)(function* () {
      return yield _this.objects.listObjectsFromIndex(_componentsIndex().IndexType.lanes);
    })();
  }

  loadLane(id) {
    var _this2 = this;

    return (0, _bluebird().coroutine)(function* () {
      if (id.isDefault()) return null; // master lane is not saved

      const filter = lane => lane.toLaneId().isEqual(id);

      const hash = _this2.objects.getHashFromIndex(_componentsIndex().IndexType.lanes, filter);

      if (!hash) return null;
      return yield _this2.objects.load(new (_objects().Ref)(hash));
    })();
  }

  saveLane(laneObject, persist = false) {
    var _this3 = this;

    return (0, _bluebird().coroutine)(function* () {
      _this3.objects.add(laneObject);

      if (persist) yield _this3.objects.persist();
    })();
  }

  getCurrentLaneName() {
    return this.scopeJson.lanes.current;
  }

  getCurrentLaneObject() {
    var _this4 = this;

    return (0, _bluebird().coroutine)(function* () {
      return _this4.loadLane(_laneId().LocalLaneId.from(_this4.getCurrentLaneName() || _constants().DEFAULT_LANE));
    })();
  }

  setCurrentLane(laneName) {
    this.scopeJson.setCurrentLane(laneName);
  }

  getLocalTrackedLaneByRemoteName(remoteLane, remoteScope) {
    const trackedLane = this.scopeJson.lanes.tracking.find(t => t.remoteLane === remoteLane && t.remoteScope === remoteScope);
    return trackedLane ? trackedLane.localLane : null;
  }

  getRemoteTrackedDataByLocalLane(localLane) {
    return this.scopeJson.lanes.tracking.find(t => t.localLane === localLane);
  }

  trackLane(trackLaneData) {
    this.scopeJson.trackLane(trackLaneData);
  }

  removeLanes(scope, lanes, force) {
    var _this5 = this;

    return (0, _bluebird().coroutine)(function* () {
      const existingLanes = yield _this5.listLanes();
      const lanesToRemove = lanes.map(laneName => {
        if (laneName === _constants().DEFAULT_LANE) throw new (_generalError().default)(`unable to remove the default lane "${_constants().DEFAULT_LANE}"`);
        if (laneName === _this5.getCurrentLaneName()) throw new (_generalError().default)(`unable to remove the currently used lane "${laneName}"`);
        const existingLane = existingLanes.find(l => l.name === laneName);
        if (!existingLane) throw new (_generalError().default)(`lane ${laneName} was not found in scope`);
        return existingLane;
      });

      if (!force) {
        yield Promise.all(lanesToRemove.map( /*#__PURE__*/function () {
          var _ref = (0, _bluebird().coroutine)(function* (laneObj) {
            const isFullyMerged = yield laneObj.isFullyMerged(scope);

            if (!isFullyMerged) {
              // @todo: this error comes pretty ugly to the client when removing from a lane from remote using ssh
              throw new (_generalError().default)(`unable to remove ${laneObj.name} lane, it is not fully merged. to disregard this error, please use --force flag`);
            }
          });

          return function (_x) {
            return _ref.apply(this, arguments);
          };
        }()));
      }

      _this5.objects.removeManyObjects(lanesToRemove.map(l => l.hash()));

      yield _this5.objects.persist();
      return lanes;
    })();
  }

  getLanesData(scope, name, mergeData) {
    var _this6 = this;

    return (0, _bluebird().coroutine)(function* () {
      const getLaneDataOfLane = /*#__PURE__*/function () {
        var _ref2 = (0, _bluebird().coroutine)(function* (laneObject) {
          const laneName = laneObject.name;

          const trackingData = _this6.getRemoteTrackedDataByLocalLane(laneName);

          return {
            name: laneName,
            remote: trackingData ? `${trackingData.remoteScope}${_constants().LANE_REMOTE_DELIMITER}${trackingData.remoteLane}` : null,
            components: laneObject.components.map(c => ({
              id: c.id,
              head: c.head.toString()
            })),
            isMerged: mergeData ? yield laneObject.isFullyMerged(scope) : null
          };
        });

        return function getLaneDataOfLane(_x2) {
          return _ref2.apply(this, arguments);
        };
      }();

      if (name) {
        const laneObject = yield _this6.loadLane(_laneId().default.from(name));
        if (!laneObject) throw new (_generalError().default)(`lane "${name}" was not found`);
        return [yield getLaneDataOfLane(laneObject)];
      }

      const lanesObjects = yield _this6.listLanes();
      const lanes = yield Promise.all(lanesObjects.map(laneObject => getLaneDataOfLane(laneObject)));
      return lanes;
    })();
  }

}

exports.default = Lanes;