/// <reference types="node" />
import { Scope } from '..';
import { BitId } from '../../bit-id';
import LaneId, { RemoteLaneId } from '../../lane-id/lane-id';
import { BitObject, Ref, Repository } from '../objects';
export declare type LaneProps = {
    name: string;
    scope?: string;
    components?: LaneComponent[];
    hash: string;
};
export declare type LaneComponent = {
    id: BitId;
    head: Ref;
};
export default class Lane extends BitObject {
    name: string;
    scope?: string;
    remoteLaneId?: RemoteLaneId;
    components: LaneComponent[];
    _hash: string;
    constructor(props: LaneProps);
    id(): string;
    hash(): Ref;
    refs(): Ref[];
    validateBeforePersisting(str: string): void;
    toObject(): any;
    static from(props: LaneProps): Lane;
    static create(name: string): Lane;
    static parse(contents: string, hash: string): Lane;
    toBuffer(pretty?: boolean): Buffer;
    addComponent(component: LaneComponent): void;
    removeComponent(id: BitId): boolean;
    getComponentByName(bitId: BitId): LaneComponent | undefined;
    getComponent(id: BitId): LaneComponent | undefined;
    getComponentHead(bitId: BitId): Ref | null;
    setLaneComponents(laneComponents: LaneComponent[]): void;
    isFullyMerged(scope: Scope): Promise<boolean>;
    getMergedAndUnmergedIds(scope: Scope): Promise<{
        merged: BitId[];
        unmerged: BitId[];
    }>;
    toBitIds(): BitId[];
    toLaneId(): LaneId;
    collectObjectsById(repo: Repository): Promise<Array<{
        id: BitId;
        objects: BitObject[];
    }>>;
    validate(): void;
}
