import LaneId from '../../lane-id/lane-id';
import { BitObject, Ref } from '.';
interface IndexItem {
    hash: string;
    toIdentifierString(): string;
}
export declare class ComponentItem implements IndexItem {
    id: {
        scope: string | null;
        name: string;
    };
    isSymlink: boolean;
    hash: string;
    constructor(id: {
        scope: string | null;
        name: string;
    }, isSymlink: boolean, hash: string);
    toIdentifierString(): string;
}
export declare class LaneItem implements IndexItem {
    id: {
        name: string;
    };
    hash: string;
    constructor(id: {
        name: string;
    }, hash: string);
    toIdentifierString(): string;
    toLaneId(): LaneId;
}
export declare enum IndexType {
    components = "components",
    lanes = "lanes"
}
declare type Index = {
    [IndexType.components]: ComponentItem[];
    [IndexType.lanes]: LaneItem[];
};
export default class ScopeIndex {
    indexPath: string;
    index: Index;
    constructor(indexPath: string, index?: Index);
    static load(basePath: string): Promise<ScopeIndex>;
    static create(basePath: string): ScopeIndex;
    static reset(basePath: string): Promise<void>;
    write(): Promise<void>;
    getAll(): IndexItem[];
    getHashes(indexType: IndexType): string[];
    getHashesByQuery(indexType: IndexType, filter: Function): string[];
    getHashesIncludeSymlinks(): string[];
    addMany(bitObjects: BitObject[]): boolean;
    addOne(bitObject: BitObject): boolean;
    removeMany(refs: Ref[]): boolean;
    removeOne(hash: string): boolean;
    deleteFile(): Promise<void>;
    getPath(): string;
    isFileOnBitHub(): boolean;
    find(hash: string): IndexItem | null;
    _exist(hash: string): boolean;
    static _composePath(basePath: string): string;
}
export {};
