"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

require("core-js/modules/es.array.iterator");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.IndexType = exports.LaneItem = exports.ComponentItem = void 0;

function _bluebird() {
  const data = require("bluebird");

  _bluebird = function () {
    return data;
  };

  return data;
}

function _defineProperty2() {
  const data = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

  _defineProperty2 = function () {
    return data;
  };

  return data;
}

function _fsExtra() {
  const data = _interopRequireDefault(require("fs-extra"));

  _fsExtra = function () {
    return data;
  };

  return data;
}

function path() {
  const data = _interopRequireWildcard(require("path"));

  path = function () {
    return data;
  };

  return data;
}

function _ramda() {
  const data = _interopRequireDefault(require("ramda"));

  _ramda = function () {
    return data;
  };

  return data;
}

function _laneId() {
  const data = _interopRequireDefault(require("../../lane-id/lane-id"));

  _laneId = function () {
    return data;
  };

  return data;
}

function _logger() {
  const data = _interopRequireDefault(require("../../logger/logger"));

  _logger = function () {
    return data;
  };

  return data;
}

function _invalidIndexJson() {
  const data = _interopRequireDefault(require("../exceptions/invalid-index-json"));

  _invalidIndexJson = function () {
    return data;
  };

  return data;
}

function _models() {
  const data = require("../models");

  _models = function () {
    return data;
  };

  return data;
}

function _lane() {
  const data = _interopRequireDefault(require("../models/lane"));

  _lane = function () {
    return data;
  };

  return data;
}

/* eslint max-classes-per-file: 0 */
const COMPONENTS_INDEX_FILENAME = 'index.json';

class ComponentItem {
  constructor(id, isSymlink, hash) {
    this.id = id;
    this.isSymlink = isSymlink;
    this.hash = hash;
  }

  toIdentifierString() {
    const scope = this.id.scope ? `${this.id.scope}/` : '';
    return `component "${scope}${this.id.name}"`;
  }

}

exports.ComponentItem = ComponentItem;

class LaneItem {
  constructor(id, hash) {
    this.id = id;
    this.hash = hash;
  }

  toIdentifierString() {
    return `lane "${this.id.name}"`;
  }

  toLaneId() {
    return new (_laneId().default)({
      name: this.id.name
    });
  }

}

exports.LaneItem = LaneItem;
let IndexType;
exports.IndexType = IndexType;

(function (IndexType) {
  IndexType["components"] = "components";
  IndexType["lanes"] = "lanes";
})(IndexType || (exports.IndexType = IndexType = {}));

class ScopeIndex {
  constructor(indexPath, index = {
    [IndexType.components]: [],
    [IndexType.lanes]: []
  }) {
    (0, _defineProperty2().default)(this, "indexPath", void 0);
    (0, _defineProperty2().default)(this, "index", void 0);
    this.indexPath = indexPath;
    this.index = index;
  }

  static load(basePath) {
    var _this = this;

    return (0, _bluebird().coroutine)(function* () {
      const indexPath = _this._composePath(basePath);

      try {
        const indexRaw = yield _fsExtra().default.readJson(indexPath);

        const getIndexWithBackwardCompatibility = () => {
          if (Array.isArray(indexRaw)) {
            return {
              [IndexType.components]: indexRaw,
              [IndexType.lanes]: []
            };
          }

          return indexRaw;
        };

        const indexObject = getIndexWithBackwardCompatibility();
        const index = {
          [IndexType.components]: indexObject[IndexType.components].map(c => new ComponentItem(c.id, c.isSymlink, c.hash)),
          [IndexType.lanes]: indexObject[IndexType.lanes].map(l => new LaneItem(l.id, l.hash))
        };
        return new ScopeIndex(indexPath, index);
      } catch (err) {
        if (err.message.includes('Unexpected token')) {
          throw new (_invalidIndexJson().default)(indexPath, err.message);
        }

        throw err;
      }
    })();
  }

  static create(basePath) {
    const indexPath = this._composePath(basePath);

    return new ScopeIndex(indexPath);
  }

  static reset(basePath) {
    var _this2 = this;

    return (0, _bluebird().coroutine)(function* () {
      const indexPath = _this2._composePath(basePath);

      _logger().default.debug(`ComponentsIndex, deleting the index file at ${indexPath}`);

      yield _fsExtra().default.remove(indexPath);
    })();
  }

  write() {
    var _this3 = this;

    return (0, _bluebird().coroutine)(function* () {
      return _fsExtra().default.writeJson(_this3.indexPath, _this3.index, {
        spaces: 2
      });
    })();
  }

  getAll() {
    return _ramda().default.flatten(Object.values(this.index));
  }

  getHashes(indexType) {
    // @ts-ignore how to tell TS that all this.index.prop are array?
    return this.index[indexType].map(indexItem => indexItem.hash);
  }

  getHashesByQuery(indexType, filter) {
    // @ts-ignore how to tell TS that all this.index.prop are array?
    return this.index[indexType].filter(filter).map(indexItem => indexItem.hash);
  }

  getHashesIncludeSymlinks() {
    return this.index.components.map(indexItem => indexItem.hash);
  }

  addMany(bitObjects) {
    const added = bitObjects.map(bitObject => this.addOne(bitObject));
    return added.some(oneAdded => oneAdded); // return true if one of the objects was added
  }

  addOne(bitObject) {
    if (!(bitObject instanceof _models().ModelComponent) && !(bitObject instanceof _models().Symlink) && !(bitObject instanceof _lane().default)) return false;
    const hash = bitObject.hash().toString();
    if (this._exist(hash)) return false;

    if (bitObject instanceof _models().ModelComponent || bitObject instanceof _models().Symlink) {
      const componentItem = new ComponentItem({
        scope: bitObject.scope || null,
        name: bitObject.name
      }, bitObject instanceof _models().Symlink, hash);
      this.index.components.push(componentItem);
    } else if (bitObject instanceof _lane().default) {
      const laneItem = new LaneItem({
        name: bitObject.name
      }, hash);
      this.index.lanes.push(laneItem);
    }

    return true;
  }

  removeMany(refs) {
    const removed = refs.map(ref => this.removeOne(ref.toString()));
    return removed.some(removedOne => removedOne); // return true if one of the objects was removed
  }

  removeOne(hash) {
    for (const entity of Object.keys(IndexType)) {
      const found = this.index[entity].find(indexItem => indexItem.hash === hash);

      if (found) {
        this.index[entity] = _ramda().default.without([found], this.index[entity]);
        return true;
      }
    }

    return false;
  }

  deleteFile() {
    var _this4 = this;

    return (0, _bluebird().coroutine)(function* () {
      _logger().default.debug(`ComponentsIndex, deleting the index file at ${_this4.indexPath}`);

      yield _fsExtra().default.remove(_this4.indexPath);
    })();
  }

  getPath() {
    return this.indexPath;
  }
  /**
   * it's obviously not accurate. a local path might include 'bithub' as part of the path as well.
   * however, it's needed only for suppressing the error message when the indexJson is outdate,
   * so if it happens on a local scope it's okay.
   * for other purposes, don't rely on this.
   */


  isFileOnBitHub() {
    return this.indexPath.includes('/bithub/');
  }

  find(hash) {
    for (const entity of Object.keys(IndexType)) {
      const found = this.index[entity].find(indexItem => indexItem.hash === hash);
      if (found) return found;
    }

    return null;
  }

  _exist(hash) {
    return Boolean(this.find(hash));
  }

  static _composePath(basePath) {
    return path().join(basePath, COMPONENTS_INDEX_FILENAME);
  }

}

exports.default = ScopeIndex;