"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

require("core-js/modules/es.array.iterator");

require("core-js/modules/es.promise");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

function _bluebird() {
  const data = require("bluebird");

  _bluebird = function () {
    return data;
  };

  return data;
}

function _defineProperty2() {
  const data = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

  _defineProperty2 = function () {
    return data;
  };

  return data;
}

function _ramda() {
  const data = _interopRequireDefault(require("ramda"));

  _ramda = function () {
    return data;
  };

  return data;
}

function _constants() {
  const data = require("../../constants");

  _constants = function () {
    return data;
  };

  return data;
}

function _manipulateDir() {
  const data = require("../../consumer/component-ops/manipulate-dir");

  _manipulateDir = function () {
    return data;
  };

  return data;
}

function _artifactFiles() {
  const data = require("../../consumer/component/sources/artifact-files");

  _artifactFiles = function () {
    return data;
  };

  return data;
}

function _generalError() {
  const data = _interopRequireDefault(require("../../error/general-error"));

  _generalError = function () {
    return data;
  };

  return data;
}

function _logger() {
  const data = _interopRequireDefault(require("../../logger/logger"));

  _logger = function () {
    return data;
  };

  return data;
}

function _versionParser() {
  const data = require("../../version/version-parser");

  _versionParser = function () {
    return data;
  };

  return data;
}

function _traverseVersions() {
  const data = require("../component-ops/traverse-versions");

  _traverseVersions = function () {
    return data;
  };

  return data;
}

function _exceptions() {
  const data = require("../exceptions");

  _exceptions = function () {
    return data;
  };

  return data;
}

function _componentNeedsUpdate() {
  const data = _interopRequireDefault(require("../exceptions/component-needs-update"));

  _componentNeedsUpdate = function () {
    return data;
  };

  return data;
}

function _unmergedComponents() {
  const data = _interopRequireDefault(require("../lanes/unmerged-components"));

  _unmergedComponents = function () {
    return data;
  };

  return data;
}

function _models() {
  const data = require("../models");

  _models = function () {
    return data;
  };

  return data;
}

function _objects() {
  const data = require("../objects");

  _objects = function () {
    return data;
  };

  return data;
}

class SourceRepository {
  constructor(scope) {
    (0, _defineProperty2().default)(this, "scope", void 0);
    this.scope = scope;
  }

  objects() {
    return this.scope.objects;
  }

  getMany(ids) {
    _logger().default.debug(`sources.getMany, Ids: ${ids.join(', ')}`);

    return Promise.all(ids.map(id => {
      return this.get(id).then(component => {
        return {
          id,
          component
        };
      });
    }));
  }

  get(bitId) {
    var _this = this;

    return (0, _bluebird().coroutine)(function* () {
      const component = _models().ModelComponent.fromBitId(bitId);

      const foundComponent = yield _this._findComponent(component);

      if (foundComponent && bitId.hasVersion()) {
        // @ts-ignore
        const isSnap = (0, _versionParser().isHash)(bitId.version);
        const msg = `found ${bitId.toStringWithoutVersion()}, however version ${bitId.getVersion().versionNum}`; // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!

        if (isSnap) {
          // @ts-ignore
          const snap = yield _this.objects().load(new (_objects().Ref)(bitId.version));

          if (!snap) {
            _logger().default.debugAndAddBreadCrumb('sources.get', `${msg} object was not found on the filesystem`);

            return undefined;
          }

          return foundComponent;
        } // @ts-ignore


        if (!foundComponent.hasTag(bitId.version)) {
          _logger().default.debugAndAddBreadCrumb('sources.get', `${msg} is not in the component versions array`);

          return undefined;
        } // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!


        const version = yield _this.objects().load(foundComponent.versions[bitId.version]);

        if (!version) {
          _logger().default.debugAndAddBreadCrumb('sources.get', `${msg} object was not found on the filesystem`);

          return undefined;
        }
      }

      return foundComponent;
    })();
  }

  _findComponent(component) {
    var _this2 = this;

    return (0, _bluebird().coroutine)(function* () {
      try {
        const foundComponent = yield _this2.objects().load(component.hash());

        if (foundComponent instanceof _models().Symlink) {
          return _this2._findComponentBySymlink(foundComponent);
        } // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!


        if (foundComponent) return foundComponent;
      } catch (err) {
        _logger().default.error(`findComponent got an error ${err}`);
      }

      _logger().default.debug(`failed finding a component ${component.id()} with hash: ${component.hash().toString()}`);

      return undefined;
    })();
  }

  _findComponentBySymlink(symlink) {
    var _this3 = this;

    return (0, _bluebird().coroutine)(function* () {
      const realComponentId = symlink.getRealComponentId();

      const realModelComponent = _models().ModelComponent.fromBitId(realComponentId);

      const foundComponent = yield _this3.objects().load(realModelComponent.hash());

      if (!foundComponent) {
        throw new Error(`error: found a symlink object "${symlink.id()}" that references to a non-exist component "${realComponentId.toString()}".
if you have the steps to reproduce the issue, please open a Github issue with the details.
to quickly fix the issue, please delete the object at "${_this3.objects().objectPath(symlink.hash())}"`);
      } // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!


      return foundComponent;
    })();
  }

  getObjects(id) {
    return this.get(id).then(component => {
      if (!component) throw new (_exceptions().ComponentNotFound)(id.toString());
      return component.collectObjects(this.objects());
    });
  }

  findOrAddComponent(props) {
    const comp = _models().ModelComponent.from(props);

    return this._findComponent(comp).then(component => {
      if (!component) return comp;
      return component;
    });
  }

  modifyCIProps({
    source,
    ciProps
  }) {
    const objectRepo = this.objects(); // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!

    return this.findOrAddComponent(source).then(component => {
      return component.loadVersion(component.latest(), objectRepo).then(version => {
        // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!
        version.setCIProps(ciProps);
        return objectRepo._writeOne(version);
      });
    });
  }

  modifySpecsResults({
    source,
    specsResults
  }) {
    const objectRepo = this.objects(); // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!

    return this.findOrAddComponent(source).then(component => {
      return component.loadVersion(component.latest(), objectRepo).then(version => {
        version.setSpecsResults(specsResults);
        return objectRepo._writeOne(version);
      });
    });
  } // TODO: This should treat dist as an array


  updateDist({
    source
  }) {
    const objectRepo = this.objects(); // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!

    return this.findOrAddComponent(source).then(component => {
      return component.loadVersion(component.latest(), objectRepo).then(version => {
        // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!
        // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!
        const dist = source.dist ? _models().Source.from(Buffer.from(source.dist.toString())) : undefined;
        version.setDist(dist);
        objectRepo.add(dist).add(version);
        return objectRepo.persist();
      });
    });
  }

  transformArtifactsFromVinylToSource(artifactsFiles) {
    const artifacts = [];
    artifactsFiles.forEach(artifactFiles => {
      const artifactsSource = _artifactFiles().ArtifactFiles.fromVinylsToSources(artifactFiles.vinyls);

      if (artifactsSource.length) artifactFiles.populateRefsFromSources(artifactsSource);
      artifacts.push(...artifactsSource);
    });
    return artifacts;
  }
  /**
   * given a consumer-component object, returns the Version representation.
   * useful for saving into the model or calculation the hash for comparing with other Version object.
   * among other things, it reverts the path manipulation that was done when a component was loaded
   * from the filesystem. it adds the originallySharedDir and strip the wrapDir.
   *
   * warning: Do not change anything on the consumerComponent instance! Only use its clone.
   *
   * @see model-components.toConsumerComponent() for the opposite action. (converting Version to
   * ConsumerComponent).
   */


  consumerComponentToVersion({
    consumerComponent,
    consumer
  }) {
    return (0, _bluebird().coroutine)(function* () {
      const clonedComponent = consumerComponent.clone();

      const setEol = files => {
        if (!files) return null;
        const result = files.map(file => {
          // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!
          file.file = file.toSourceAsLinuxEOL();
          return file;
        });
        return result;
      };

      const manipulateDirs = pathStr => {
        return (0, _manipulateDir().revertDirManipulationForPath)(pathStr, clonedComponent.originallySharedDir, clonedComponent.wrapDir);
      };

      const files = consumerComponent.files.map(file => {
        return {
          name: file.basename,
          relativePath: manipulateDirs(file.relative),
          file: file.toSourceAsLinuxEOL(),
          test: file.test
        };
      }); // @todo: is this the best way to find out whether a compiler is set?

      const isCompileSet = Boolean(consumerComponent.compiler || clonedComponent.extensions.some(e => e.name === _constants().Extensions.compiler || e.name === 'bit.core/compile' || e.name === _constants().Extensions.envs));
      const {
        dists,
        mainDistFile
      } = clonedComponent.dists.toDistFilesModel(consumer, consumerComponent.originallySharedDir, isCompileSet);
      const compilerFiles = setEol(_ramda().default.path(['compiler', 'files'], consumerComponent));
      const testerFiles = setEol(_ramda().default.path(['tester', 'files'], consumerComponent));
      clonedComponent.mainFile = manipulateDirs(clonedComponent.mainFile);
      clonedComponent.getAllDependencies().forEach(dependency => {
        // ignoreVersion because when persisting the tag is higher than currently exist in .bitmap
        const depFromBitMap = consumer.bitMap.getComponentIfExist(dependency.id, {
          ignoreVersion: true
        });
        dependency.relativePaths.forEach(relativePath => {
          if (!relativePath.isCustomResolveUsed) {
            // for isCustomResolveUsed it was never stripped
            relativePath.sourceRelativePath = manipulateDirs(relativePath.sourceRelativePath);
          }

          if (depFromBitMap && depFromBitMap.origin !== _constants().COMPONENT_ORIGINS.AUTHORED) {
            // when a dependency is not authored, we need to also change the
            // destinationRelativePath, which is the path written in the link file, however, the
            // dir manipulation should be according to this dependency component, not the
            // consumerComponent passed to this function
            relativePath.destinationRelativePath = (0, _manipulateDir().revertDirManipulationForPath)(relativePath.destinationRelativePath, depFromBitMap.originallySharedDir, depFromBitMap.wrapDir);
          }
        });
      });
      clonedComponent.overrides.addOriginallySharedDir(clonedComponent.originallySharedDir);

      const version = _models().Version.fromComponent({
        component: clonedComponent,
        files: files,
        dists,
        // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!
        mainDistFile
      }); // $FlowFixMe it's ok to override the pendingVersion attribute


      consumerComponent.pendingVersion = version; // helps to validate the version against the consumer-component

      return {
        version,
        files,
        dists,
        compilerFiles,
        testerFiles
      };
    })();
  }

  enrichSource(consumerComponent) {
    var _this4 = this;

    return (0, _bluebird().coroutine)(function* () {
      const objectRepo = _this4.objects();

      const component = yield _this4.findOrAddComponent(consumerComponent);
      const version = yield component.loadVersion(consumerComponent.id.version, objectRepo);
      const artifactFiles = (0, _artifactFiles().getArtifactsFiles)(consumerComponent.extensions);

      const artifacts = _this4.transformArtifactsFromVinylToSource(artifactFiles);

      version.extensions = consumerComponent.extensions;
      artifacts.forEach(file => objectRepo.add(file.source));
      objectRepo.add(version);
      return consumerComponent;
    })();
  }

  addSource({
    source,
    consumer,
    lane,
    resolveUnmerged = false
  }) {
    var _this5 = this;

    return (0, _bluebird().coroutine)(function* () {
      const objectRepo = _this5.objects(); // if a component exists in the model, add a new version. Otherwise, create a new component on the model
      // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!


      const component = yield _this5.findOrAddComponent(source);
      const unmergedComponent = consumer.scope.objects.unmergedComponents.getEntry(component.name);

      if (unmergedComponent && !unmergedComponent.resolved && !resolveUnmerged) {
        throw new (_generalError().default)(`unable to snap/tag "${component.name}", it is unmerged with conflicts. please run "bit merge <id> --resolve"`);
      }

      const artifactFiles = (0, _artifactFiles().getArtifactsFiles)(source.extensions);

      const artifacts = _this5.transformArtifactsFromVinylToSource(artifactFiles); // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!


      const {
        version,
        files,
        dists,
        compilerFiles,
        testerFiles
      } = yield _this5.consumerComponentToVersion({
        consumerComponent: source,
        consumer
      });
      objectRepo.add(version);
      if (!source.version) throw new Error(`addSource expects source.version to be set`);
      component.addVersion(version, source.version, lane, objectRepo);

      if (unmergedComponent) {
        version.addParent(unmergedComponent.head);
        version.log.message = version.log.message ? version.log.message : _unmergedComponents().default.buildSnapMessage(unmergedComponent);
        consumer.scope.objects.unmergedComponents.removeComponent(component.name);
      }

      objectRepo.add(component);
      files.forEach(file => objectRepo.add(file.file));
      if (dists) dists.forEach(dist => objectRepo.add(dist.file));
      if (compilerFiles) compilerFiles.forEach(file => objectRepo.add(file.file));
      if (testerFiles) testerFiles.forEach(file => objectRepo.add(file.file));
      if (artifacts) artifacts.forEach(file => objectRepo.add(file.source));
      return component;
    })();
  }

  putModelComponent(component) {
    const repo = this.objects();
    repo.add(component);
  }

  put({
    component,
    objects
  }) {
    _logger().default.debug(`sources.put, id: ${component.id()}, versions: ${component.listVersions().join(', ')}`);

    const repo = this.objects();
    repo.add(component);

    const isObjectShouldBeAdded = obj => {
      // don't add a component if it's already exist locally with more versions
      if (obj instanceof _models().ModelComponent) {
        const loaded = repo.loadSync(obj.hash(), false);

        if (loaded) {
          // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!
          if (Object.keys(loaded.versions) > Object.keys(obj.versions)) {
            return false;
          }
        }
      }

      return true;
    };

    objects.forEach(obj => {
      if (isObjectShouldBeAdded(obj)) repo.add(obj);
    });
    return component;
  }

  putObjects(objects) {
    const repo = this.objects();
    objects.forEach(obj => repo.add(obj));
  }
  /**
   * remove specified component versions from component.
   * if all versions of a component were deleted, delete also the component.
   * it doesn't persist anything to the filesystem.
   * (repository.persist() needs to be called at the end of the operation)
   */


  removeComponentVersions(component, versions, allVersionsObjects) {
    _logger().default.debug(`removeComponentVersion, component ${component.id()}, versions ${versions.join(', ')}`);

    const objectRepo = this.objects();
    versions.forEach(version => {
      const ref = component.removeVersion(version);
      const refStr = ref.toString();
      const versionObject = allVersionsObjects.find(v => v.hash().isEqual(ref));
      if (!versionObject) throw new Error(`removeComponentVersions failed finding a version object of ${refStr}`); // update the snap head if needed

      if (component.getHeadStr() === refStr) {
        if (versionObject.parents.length > 1) throw new Error(`removeComponentVersions found multiple parents for a local (un-exported) version ${version} of ${component.id()}`);
        const head = versionObject.parents.length === 1 ? versionObject.parents[0] : undefined;
        component.setHead(head);
      } // update other versions parents if they point to the deleted version


      allVersionsObjects.forEach(obj => {
        if (obj.hasParent(ref)) {
          obj.removeParent(ref);
          objectRepo.add(obj);
        }
      });
      objectRepo.removeObject(ref);
    });

    if (component.versionArray.length || component.hasHead()) {
      objectRepo.add(component); // add the modified component object
    } else {
      // @todo: make sure not to delete the component when it has snaps but not versions!
      // if all versions were deleted, delete also the component itself from the model
      objectRepo.removeObject(component.hash());
    }

    objectRepo.unmergedComponents.removeComponent(component.name);
  }
  /**
   * @see this.removeComponent()
   *
   */


  removeComponentById(bitId) {
    var _this6 = this;

    return (0, _bluebird().coroutine)(function* () {
      _logger().default.debug(`sources.removeComponentById: ${bitId.toString()}`);

      const component = yield _this6.get(bitId);
      if (!component) return;

      _this6.removeComponent(component);
    })();
  }
  /**
   * remove all versions objects of the component from local scope.
   * if deepRemove is true, it removes also the refs associated with the removed versions.
   * finally, it removes the component object itself
   * it doesn't physically delete from the filesystem.
   * the actual delete is done at a later phase, once Repository.persist() is called.
   *
   * @param {ModelComponent} component
   * @param {boolean} [deepRemove=false] - whether remove all the refs or only the version array
   */


  removeComponent(component) {
    const repo = this.objects();

    _logger().default.debug(`sources.removeComponent: removing a component ${component.id()} from a local scope`);

    const objectRefs = component.versionArray;
    objectRefs.push(component.hash());
    repo.removeManyObjects(objectRefs);
    repo.unmergedComponents.removeComponent(component.name);
  }
  /**
   * merge the existing component with the data from the incoming component
   * here, we assume that there is no conflict between the two, otherwise, this.merge() would throw
   * a MergeConflict exception.
   */


  mergeTwoComponentsObjects(existingComponent, incomingComponent, existingComponentTagsAndSnaps, incomingComponentTagsAndSnaps) {
    // the base component to save is the existingComponent because it might contain local data that
    // is not available in the remote component, such as the "state" property.
    const mergedComponent = existingComponent;
    const mergedVersions = []; // in case the existing version hash is different than incoming version hash, use the incoming
    // version because we hold the incoming component from a remote as the source of truth

    Object.keys(existingComponent.versions).forEach(existingVersion => {
      if (incomingComponent.versions[existingVersion] && existingComponent.versions[existingVersion].toString() !== incomingComponent.versions[existingVersion].toString()) {
        mergedComponent.versions[existingVersion] = incomingComponent.versions[existingVersion];
        mergedVersions.push(existingVersion);
      }
    }); // in case the incoming component has versions that are not in the existing component, copy them

    Object.keys(incomingComponent.versions).forEach(incomingVersion => {
      if (!existingComponent.versions[incomingVersion]) {
        mergedComponent.versions[incomingVersion] = incomingComponent.versions[incomingVersion];
        mergedVersions.push(incomingVersion);
      }
    });

    if (incomingComponent.hasHead()) {
      const mergedSnaps = incomingComponentTagsAndSnaps.filter(tagOrSnap => !existingComponentTagsAndSnaps.includes(tagOrSnap) && !mergedVersions.includes(tagOrSnap));
      mergedVersions.push(...mergedSnaps);
    }

    return {
      mergedComponent,
      mergedVersions
    };
  }
  /**
   * Adds the objects into scope.object array, in-memory. It doesn't save anything to the file-system.
   *
   * When this function gets called originally from import command, the 'local' parameter is true. Otherwise, if it was
   * originated from export command, it'll be false.
   * If the 'local' is true and the existing component wasn't changed locally, it doesn't check for
   * discrepancies, but simply override the existing component.
   * In this context, "discrepancy" means, same version but different hashes.
   * When using import command, it makes sense to override a component in case of discrepancies because the source of
   * truth should be the remote scope from where the import fetches the component.
   * When the same component has different versions in the remote and the local, it merges the two
   * by calling this.mergeTwoComponentsObjects().
   *
   * when dealing with lanes, exporting/importing lane's components, this function doesn't do much
   * if any. that's because the head is not saved on the ModelComponent but on the lane object.
   * to rephrase with other words,
   * this function merges an incoming modelComponent with an existing modelComponent, so if all
   * changes where done on a lane, this function will not do anything because modelComponent
   * hasn't changed.
   */


  merge(component, versionObjects, inScope = false, local = true) {
    var _this7 = this;

    return (0, _bluebird().coroutine)(function* () {
      if (inScope) component.scope = _this7.scope.name;
      const existingComponent = yield _this7._findComponent(component); // @ts-ignore
      // const versionObjects: Version[] = objects.filter((o) => o instanceof Version);
      // don't throw if not found because on export not all objects are sent to the remote

      const allHashes = yield (0, _traverseVersions().getAllVersionHashesByVersionsObjects)(component, versionObjects, false);
      const tagsAndSnaps = component.switchHashesWithTagsIfExist(allHashes);

      if (!existingComponent) {
        _this7.putModelComponent(component);

        return {
          mergedComponent: component,
          mergedVersions: tagsAndSnaps
        };
      }

      const existingComponentHead = existingComponent.getHead();
      const existingHeadIsMissingInIncomingComponent = component.hasHead() && existingComponentHead && !allHashes.find(ref => ref.isEqual(existingComponentHead));

      if (!local && existingHeadIsMissingInIncomingComponent && component.compatibleWith(existingComponent, local) // otherwise, it should throw MergeConflict below
      ) {
          // @ts-ignore
          throw new (_componentNeedsUpdate().default)(component.id(), existingComponentHead.toString());
        } // @todo lanes: should we pass the local lane to `isLocallyChanged`?


      const locallyChanged = yield existingComponent.isLocallyChanged();

      if (local && !locallyChanged || component.compatibleWith(existingComponent, local)) {
        _logger().default.debug(`sources.merge component ${component.id()}`);

        const repo = _this7.objects();

        const existingComponentHashes = yield (0, _traverseVersions().getAllVersionHashes)(existingComponent, repo, false);
        const existingComponentTagsAndSnaps = existingComponent.switchHashesWithTagsIfExist(existingComponentHashes);

        const {
          mergedComponent,
          mergedVersions
        } = _this7.mergeTwoComponentsObjects(existingComponent, component, existingComponentTagsAndSnaps, tagsAndSnaps);

        const componentHead = component.getHead();

        if (componentHead) {
          // when importing (local), do not override the head
          if (!local || !existingHeadIsMissingInIncomingComponent) {
            mergedComponent.setHead(componentHead);
          }

          if (local) mergedComponent.remoteHead = componentHead;
        }

        _this7.putModelComponent(mergedComponent);

        return {
          mergedComponent,
          mergedVersions
        };
      }

      const conflictVersions = component.diffWith(existingComponent, local);
      throw new (_exceptions().MergeConflict)(component.id(), conflictVersions);
    })();
  }
  /**
   * the merge is needed only when both, local lane and remote lane have the same component with
   * a different head.
   * the different head can be a result of one component is ahead of the other (fast-forward is
   *  possible), or they both have diverged.
   *
   * 1a) fast-forward case, existing is ahead. existing has snapA => snapB, incoming has snapA.
   * we can just ignore the incoming.
   *
   * 1b) fast-forward case, incoming is ahead. existing has snapA, incoming has snapA => snapB.
   * we should update the existing head according to the incoming.
   *
   * 2) true-merge case, existing has snapA => snapB, incoming has snapA => snapC.
   *
   * in case this is a remote (the incoming component comes as a result of export):
   * throw an error telling the client to pull the lane from the remote in order to merge the
   * new snaps. the client during the merge process will create a snap-merge that is going to be
   * the new head, which eventually becoming the case 1b.
   *
   * in case this is a local (the incoming component comes as a result of import):
   * do not update the lane object. only save the data on the refs/remote/lane-name.
   */


  mergeLane(lane, local) {
    var _this8 = this;

    return (0, _bluebird().coroutine)(function* () {
      const repo = _this8.objects();

      const existingLane = yield _this8.scope.loadLane(lane.toLaneId());

      if (!existingLane) {
        repo.add(lane);
      }

      const mergeResults = yield Promise.all(lane.components.map( /*#__PURE__*/function () {
        var _ref = (0, _bluebird().coroutine)(function* (component) {
          const modelComponent = yield _this8.get(component.id);

          if (!modelComponent) {
            throw new Error(`unable to merge lane ${lane.name}, the component ${component.id.toString()} was not found`);
          }

          const existingComponent = existingLane ? existingLane.components.find(c => c.id.isEqual(component.id)) : null;

          if (!existingComponent) {
            modelComponent.laneHeadLocal = component.head;
            const allVersions = yield (0, _traverseVersions().getAllVersionHashes)(modelComponent, repo);
            if (existingLane) existingLane.addComponent(component);
            return {
              mergedComponent: modelComponent,
              mergedVersions: allVersions.map(h => h.toString())
            };
          }

          if (existingComponent.head.isEqual(component.head)) {
            return {
              mergedComponent: modelComponent,
              mergedVersions: []
            };
          }

          modelComponent.laneHeadRemote = component.head;
          modelComponent.laneHeadLocal = existingComponent.head;
          yield modelComponent.setDivergeData(repo);
          const divergeResults = modelComponent.getDivergeData();

          if (divergeResults.isDiverged()) {
            if (local) {
              // do not update the local lane. later, suggest to snap-merge.
              return {
                mergedComponent: modelComponent,
                mergedVersions: []
              };
            }

            return new (_componentNeedsUpdate().default)(component.id.toString(), existingComponent.head.toString());
          }

          if (divergeResults.isRemoteAhead()) {
            existingComponent.head = component.head;
            return {
              mergedComponent: modelComponent,
              mergedVersions: divergeResults.snapsOnRemoteOnly.map(h => h.toString())
            };
          } // local is ahead, nothing to merge.


          return {
            mergedComponent: modelComponent,
            mergedVersions: []
          };
        });

        return function (_x) {
          return _ref.apply(this, arguments);
        };
      }()));
      repo.add(existingLane); // objects.forEach((obj) => repo.add(obj));

      return mergeResults;
    })();
  }

}

exports.default = SourceRepository;