"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.isStrReleaseType = isStrReleaseType;
exports.isReleaseTypeSupported = isReleaseTypeSupported;
exports.throwForUnsupportedReleaseType = throwForUnsupportedReleaseType;
exports.validateVersion = validateVersion;
exports.getValidVersionOrReleaseType = getValidVersionOrReleaseType;
exports.getLatestVersion = getLatestVersion;

function _semver() {
  const data = require("semver");

  _semver = function () {
    return data;
  };

  return data;
}

function _exceptions() {
  const data = require("../version/exceptions");

  _exceptions = function () {
    return data;
  };

  return data;
}

function _generalError() {
  const data = _interopRequireDefault(require("../error/general-error"));

  _generalError = function () {
    return data;
  };

  return data;
}

function isStrReleaseType(str) {
  const releaseTypes = ['major', 'premajor', 'minor', 'preminor', 'patch', 'prepatch', 'prerelease'];
  return releaseTypes.includes(str);
}

function isReleaseTypeSupported(str) {
  const supportedReleaseTypes = ['patch', 'minor', 'major'];
  return supportedReleaseTypes.includes(str);
}

function throwForUnsupportedReleaseType(str) {
  if (!isReleaseTypeSupported(str)) {
    throw new Error(`the release-type "${str}" is not supported`);
  }
}

function validateVersion(version) {
  if (version) {
    // it also changes to a valid string (e.g. from v1.0.0 to 1.0.0)
    const validVersion = (0, _semver().valid)(version);
    if (!validVersion) throw new (_exceptions().InvalidVersion)(version);
    if ((0, _semver().prerelease)(version)) throw new (_generalError().default)(`error: a prerelease version "${version}" is not supported`);
    return validVersion;
  }

  return undefined;
}

function getValidVersionOrReleaseType(str) {
  if (isStrReleaseType(str)) {
    const releaseType = str;
    throwForUnsupportedReleaseType(releaseType);
    return {
      releaseType
    };
  }

  const exactVersion = validateVersion(str);
  return {
    exactVersion
  };
}

function getLatestVersion(versions) {
  const max = (0, _semver().maxSatisfying)(versions, '*');
  if (!max) throw new Error(`unable to find the latest version from ${versions.join(', ')}`);
  return max;
}